/**
*deepPopulate middleware
*/
import type { Core } from '@strapi/strapi';
import { UID } from '@strapi/types';
import { contentTypes } from '@strapi/utils';
import path from 'path';
import pluralize from 'pluralize';

interface Options {
  /**
  * Fields to select when populating relations
  */
  relationalFields?: string[];
}
const { CREATED_BY_ATTRIBUTE, UPDATED_BY_ATTRIBUTE } = contentTypes.constants;

const extractPathSegment = (url: string) => url.match(/\/([^/?]+)(?:\?|$)/)?.[1] || '';

const getModelUID = (path: string): UID.Schema => {
  // Special handling for users-permissions routes
  if (path === 'users' || path === 'me') {
    return 'plugin::users-permissions.user';
  }

  // Logic for regular content types
  const singular = pluralize.singular(path);
  return `api::${singular}.${singular}` as UID.Schema;
};

const getDeepPopulate = (uid: UID.Schema, opts: Options = {}) => {
  const model = strapi.getModel(uid);

  // Add safety check
  if (!model || !model.attributes) {
    console.warn(`Model not found for UID: ${uid}`);
    return {};
  }

  const attributes = Object.entries(model.attributes);

  return attributes.reduce((acc: any, [attributeName, attribute]) => {
    switch (attribute.type) {
      case 'relation': {
        const isMorphRelation = attribute.relation.toLowerCase().startsWith('morph');
        if (isMorphRelation) {
          break;
        }

        // Ignore not visible fields other than createdBy and updatedBy
        const isVisible = contentTypes.isVisibleAttribute(model, attributeName);
        const isCreatorField = [CREATED_BY_ATTRIBUTE, UPDATED_BY_ATTRIBUTE].includes(attributeName);

        if (isVisible) {
          acc[attributeName] = { populate: "*" };
        }
        break;
      }

      case 'media': {
        acc[attributeName] = { populate: "*" };
        break;
      }

      case 'component': {
        const populate = getDeepPopulate(attribute.component, opts);
        acc[attributeName] = { populate };
        break;
      }

      case 'dynamiczone': {
        // Use fragments to populate the dynamic zone components
        const populatedComponents = (attribute.components || []).reduce(
          (acc: any, componentUID: UID.Component) => {
            acc[componentUID] = { populate: getDeepPopulate(componentUID, opts) };

            return acc;
          },
          {}
        );

        acc[attributeName] = { on: populatedComponents };
        break;
      }
      default:
        break;
    }

    return acc;
  }, {});
};

export default (config, { strapi }: { strapi: Core.Strapi }) => {
  return async (ctx, next) => {
    if (ctx.request.url.startsWith('/api/') && ctx.request.method === 'GET' && !ctx.query.populate) {
      strapi.log.info('Using custom Dynamic-Zone population Middleware...');

      let pathSegment = extractPathSegment(ctx.request.url);
      if(ctx.request.url.startsWith('/api/custom/')) {
        pathSegment = ctx.request.url.split("/")[3]
      }
      const uid = getModelUID(pathSegment);

      // @ts-ignores 
      try {
        ctx.query.populate = getDeepPopulate(uid);
      } catch (error) {
        console.error(`Error in deepPopulate middleware for path ${pathSegment}:`, error);
        // Continue without population if there's an error
      }
    }
    await next();
  };
};